#!perl -T

use 5.006;
use strict;
use warnings;
use Test::More;

BEGIN {
use_ok( 'Application::ConfigProcessor', qw( process_configs ) );
}

# Simulate options being passed in on the command line.
@ARGV = (
    '--config_1',
    'config 1 value from command line - should show.',
    '--config_2',
    'config 2 value from command line - should show.',
);

# Fake our home directory for tests
$ENV{HOME} = 'etc/home';

my $configs1 = process_configs(
    'application-defaults' => {
        'global-config-file' => 'etc/test-application/config.yaml',
        'user-config-file'   => 'test-application/config.yaml',
        'config_1'           => 'config 1 value from default hash - should not show.',
        'config_2'           => 'config 2 value from default hash - should not show.',
        'config_3'           => 'config 3 value from default hash - should not show.',
        'config_4'           => 'config 4 value from default hash - should not show.',
        'config_5'           => 'config 5 value from default hash - should not show.',
        'config_6'           => 'config 6 value from default hash - should not show.',
        'config_7'           => 'config 7 value from default hash - should show.',
        'config_8'           => 'config 8 value from default hash - should show.',
    },
);

is ( $configs1->{config_1}, 'config 1 value from command line - should show.',  'config 1 from command line' );
is ( $configs1->{config_2}, 'config 2 value from command line - should show.',  'config 2 from command line' );
is ( $configs1->{config_3}, 'config 3 value from user YAML - should show.', 'config 3 from user YAML' );
is ( $configs1->{config_4}, 'config 4 value from user YAML - should show.', 'config 4 from user YAML' );
is ( $configs1->{config_5}, 'config 5 value from global YAML - should show.', 'config 5 from global YAML' );
is ( $configs1->{config_6}, 'config 6 value from global YAML - should show.', 'config 6 from global YAML' );
is ( $configs1->{config_7}, 'config 7 value from default hash - should show.', 'config 7 from default hash' );
is ( $configs1->{config_8}, 'config 8 value from default hash - should show.', 'config 8 from default hash' );

# Reset args after 1st process configs was run.
@ARGV = (
    '--config_1',
    'config 1 value from command line - should show.',
    '--config_2',
    'config 2 value from command line - should show.',
);

my $configs2 = process_configs(
    'global-config-file-key' => 'global-config-location',
    'user-config-file-key'   => 'user-config-location',
    'application-defaults'   => {
        'global-config-location' => 'etc/test-application/config.yaml',
        'user-config-location'   => 'test-application/config.yaml',
        'config_1'               => 'config 1 value from default hash - should not show.',
        'config_2'               => 'config 2 value from default hash - should not show.',
        'config_3'               => 'config 3 value from default hash - should not show.',
        'config_4'               => 'config 4 value from default hash - should not show.',
        'config_5'               => 'config 5 value from default hash - should not show.',
        'config_6'               => 'config 6 value from default hash - should not show.',
        'config_7'               => 'config 7 value from default hash - should show.',
        'config_8'               => 'config 8 value from default hash - should show.',
    },
);

is ( $configs1->{config_1}, 'config 1 value from command line - should show.',  'config 1 from command line with non-default global/user-config-file-keys' );
is ( $configs1->{config_2}, 'config 2 value from command line - should show.',  'config 2 from command line with non-default global/user-config-file-keys' );
is ( $configs1->{config_3}, 'config 3 value from user YAML - should show.', 'config 3 from user YAML with non-default global/user-config-file-keys' );
is ( $configs1->{config_4}, 'config 4 value from user YAML - should show.', 'config 4 from user YAML with non-default global/user-config-file-keys' );
is ( $configs1->{config_5}, 'config 5 value from global YAML - should show.', 'config 5 from global YAML with non-default global/user-config-file-keys' );
is ( $configs1->{config_6}, 'config 6 value from global YAML - should show.', 'config 6 from global YAML with non-default global/user-config-file-keys' );
is ( $configs1->{config_7}, 'config 7 value from default hash - should show.', 'config 7 from default hash with non-default global/user-config-file-keys' );
is ( $configs1->{config_8}, 'config 8 value from default hash - should show.', 'config 8 from default hash with non-default global/user-config-file-keys' );

done_testing;
